import whois

import json
import toml
from dict2xml import dict2xml


def _unpack(website):
    data = whois.whois(website)
    return json.loads(str(data))


def as_json(website) -> dict:
    """Will return a JSON (dict) containing
    the whois lookup of `website`."""

    return _unpack(website)


def as_xml(website) -> str:
    """Will return a string representing a xml
    document containing the whois lookup of `website`."""

    return dict2xml(_unpack(website))


def as_toml(website):
    """Will return a string representing a TOML
    document containing the whois lookup of `website`."""

    return toml.dumps(_unpack(website))



import os
from os import path

from pathlib import Path
from shutil import rmtree


def save_to_file(data: str, out: str):
    ensure_path(out)
    with open(out, '+w') as file:
        file.write(data)


def rootname(dest: str):
    """
    Will return the root of a path.
    """
    _p = Path(dest)

    return path.join(
        _p.parents[len(p.parents) - 1],
        _p.parents[len(p.parents) - 2]
    )


def remove_path(dest: str):
    """
    Will remove the destination completely, erasing
    all the folders and files from it (if dest is
    ./a/b/c/d.xml, remove_path will act like `rm -rf
    ./a`)
    """
    _rootname = self.rootname
    rmtree(_rootname(dest), ignore_errors=True)


def ensure_path(dest: str):
    """Will ensure the folder hierarchy to a path
    (if parent folders do not exist, it will
    create them.)
    """

    if path.dirname(dest):
        os.makedirs(path.dirname(dest), exist_ok=True)



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '1.0.0'
DESCRIPTION = 'Parse whois data into program-readable formats (like JSON, XML, TOML, etc.)'
LONG_DESCRIPTION = """A package that allows you to convert whois data to computer-readable data formats (like JSON, XML, etc.)
It is convenient to make lookup programs or API integrations."""

setup(
    name="wgrep",
    version=VERSION,
    author="callope",
    author_email="callope@proton.me",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=['python-whois', 'toml', 'dict2xml'],
    keywords=['python', 'whois', 'lookup', 'api integration', 'information'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)



from . import output
from .fileutils import save_to_file
from argparse import ArgumentParser

# TODO: try to use python click here
_parser = ArgumentParser(
    "wgrep",
    description="""Save whois information into a computer-readable file format (like toml, json, xml, etc.)"""
)

_parser.add_argument(
    'target',
    metavar='target',
    help="""The target to execute lookup."""
)

_parser.add_argument(
    'output',
    metavar="output",
    default='wgrep-output',
    help="""The file to save the output."""
)

_parser.add_argument(
    'format',
    metavar="format",
    help="""The format to save the data."""
)

args = _parser.parse_args()

_lnk = args.target
_out = args.output
_fmt = args.format.lower()

cvt_table = {
    "json": output.as_json,
    "xml": output.as_xml,
    "toml": output.as_toml
}


def main():
    try:
        data = cvt_table[_fmt](_lnk)
    except KeyError:
        print(f"error: {_fmt} was not implemented")
        exit(1)
    except Exception e:
        print(f"error: {e} (unexpected error)")
        exit(1)

    save_to_file(str(data), _out)


if __name__ == "__main__":
    main()
